@@ -63,7 +63,7 @@ void bpc_attrib_backwardCompat(int writeOldStyleAttribFile, int keepOldAttribFil
 {
     if ( writeOldStyleAttribFile >= 0 ) WriteOldStyleAttribFile = writeOldStyleAttribFile;
     if ( keepOldAttribFiles >= 0 )      KeepOldAttribFiles = keepOldAttribFiles;
-    if ( BPC_LogLevel >= 2 ) {
+    if ( BPC_LogLevel >= 5 ) {
         bpc_logMsgf("bpc_attrib_backwardCompat: WriteOldStyleAttribFile = %d, KeepOldAttribFiles = %d\n",
                      WriteOldStyleAttribFile, KeepOldAttribFiles);
     }
@@ -121,7 +121,21 @@ int bpc_attrib_xattrDeleteAll(bpc_attrib_file *file)
  */
 int bpc_attrib_xattrSetValue(bpc_attrib_file *file, void *key, int keyLen, void *value, uint32 valueLen)
 {
-    bpc_attrib_xattr *xattr = bpc_attrib_xattrGet(file, key, keyLen, 1);
+    bpc_attrib_xattr *xattr;
+    char keyCopy[BPC_MAXPATHLEN];
+
+    if ( ((char*)key)[keyLen - 1] != 0x0 ) {
+        if ( keyLen >= BPC_MAXPATHLEN - 8 ) {
+            bpc_logMsgf("bpc_attrib_xattrSetValue: BOTCH: key not 0x0 terminated; too long to repair (keyLen %u)\n", keyLen);
+            return -1;
+        } else {
+            memcpy(keyCopy, key, keyLen);
+            keyCopy[keyLen++] = 0x0;
+            key = keyCopy;
+            bpc_logMsgf("bpc_attrib_xattrSetValue: BOTCH: appended 0x0 to xattr name '%s' (keyLen now %u)\n", (char*)key, keyLen);
+        }
+    }
+    xattr = bpc_attrib_xattrGet(file, key, keyLen, 1);
 
     if ( !xattr->value ) {
         /*
@@ -223,9 +237,14 @@ static void bpc_attrib_xattrListKey(bpc_attrib_xattr *xattr, xattrList_info *inf
             return;
         }
         /*
-         * keyLen already includes the \0 terminating byte
+         * confirm that keyLen includes the \0 terminating byte
          */
         memcpy(info->list + info->idx, xattr->key.key, xattr->key.keyLen);
+        if ( xattr->key.keyLen >= 1 && info->list[info->idx + xattr->key.keyLen - 1] != 0x0 ) {
+            info->list[info->idx + xattr->key.keyLen - 1] = 0x0;
+            bpc_logMsgf("bpc_attrib_xattrListKey: BOTCH: truncated xattr name '%s' to match keyLen %u\n", info->list + info->idx, xattr->key.keyLen);
+        }
+        if ( BPC_LogLevel >= 6 ) bpc_logMsgf("bpc_attrib_xattrListKey: adding %s\n", info->list + info->idx);
         info->idx += xattr->key.keyLen;
     } else {
         info->idx += xattr->key.keyLen;
@@ -369,6 +388,13 @@ void bpc_attrib_fileDeleteName(bpc_attrib_dir *dir, char *fileName)
     bpc_hashtable_nodeDelete(&dir->filesHT, file);
 }
 
+int bpc_attrib_fileIterate(bpc_attrib_dir *dir, bpc_attrib_file **file, uint *idx)
+{
+    *file = bpc_hashtable_nextEntry(&dir->filesHT, idx);
+    if ( !*file ) return -1;
+    return 0;
+}
+
 int bpc_attrib_fileCount(bpc_attrib_dir *dir)
 {
     return bpc_hashtable_entryCount(&dir->filesHT);
@@ -566,21 +592,19 @@ static int64 getVarInt_v3(uchar **bufPP, uchar *bufEnd)
 static void setVarInt(uchar **bufPP, uchar *bufEnd, int64 value)
 {
     uchar *bufP = *bufPP;
+    int maxBytes = (sizeof(value) * 8 + 6) / 7;
 
-    if ( value < 0 ) {
-        bpc_logErrf("setVarInt botch: got negative argument %ld; setting to 0\n", (long int)value);
-        value = 0;
-    }
     do {
         uchar c = value & 0x7f;
         value >>= 7;
-        if ( value ) c |= 0x80;
+        maxBytes--;
+        if ( value && maxBytes > 0 ) c |= 0x80;
         if ( bufP < bufEnd ) {
             *bufP++ = c;
         } else {
             bufP++;
         }
-    } while ( value );
+    } while ( value && maxBytes > 0 );
     *bufPP = bufP;
 }
 
@@ -644,7 +668,9 @@ uchar *bpc_attrib_buf2fileFull(bpc_attrib_file *file, uchar *bufP, uchar *bufEnd
         return NULL;
     }
     bufP += fileNameLen;
+    bpc_attrib_xattrDeleteAll(file);
     xattrNumEntries = getVarInt(&bufP, bufEnd);
+    if ( BPC_LogLevel >= 6 ) bpc_logMsgf("bpc_attrib_buf2fileFull: xattrNumEntries = %d\n", xattrNumEntries);
     bufP = bpc_attrib_buf2file(file, bufP, bufEnd, xattrNumEntries);
     return bufP;
 }
@@ -917,7 +943,7 @@ int bpc_attrib_dirRead(bpc_attrib_dir *dir, char *dirPath, char *attribFilePath,
                 return -1;
             }
 
-            fileNameLen = getVarInt(&bufP, buf + nRead);
+            fileNameLen = getVarInt_v3(&bufP, buf + nRead);
             if ( fileNameLen > 2 * BPC_MAXPATHLEN - 16 ) {
                 bpc_logErrf("bpc_attrib_dirRead: got unreasonable file name length %d\n", fileNameLen);
                 bpc_fileZIO_close(&fd);
@@ -961,6 +987,7 @@ int bpc_attrib_dirRead(bpc_attrib_dir *dir, char *dirPath, char *attribFilePath,
 typedef struct {
     uchar *bufP;
     uchar *bufEnd;
+    uint numEntries;
 } buf_info;
 
 typedef struct {
@@ -983,8 +1010,12 @@ static void bpc_attrib_xattrWrite(bpc_attrib_xattr *xattr, buf_info *info)
     setVarInt(&info->bufP, info->bufEnd, xattr->key.keyLen);
     setVarInt(&info->bufP, info->bufEnd, xattr->valueLen);
 
-    if ( info->bufP + xattr->key.keyLen <= info->bufEnd ) {
+    if ( xattr->key.keyLen >= 1 && info->bufP + xattr->key.keyLen <= info->bufEnd ) {
         memcpy(info->bufP, xattr->key.key, xattr->key.keyLen);
+        if ( info->bufP[xattr->key.keyLen - 1] != 0x0 ) {
+            info->bufP[xattr->key.keyLen - 1] = 0x0;
+            bpc_logMsgf("bpc_attrib_xattrWrite: BOTCH: truncated xattr name '%s' to match keyLen %u\n", info->bufP, xattr->key.keyLen);
+        }
     }
     info->bufP += xattr->key.keyLen;
 
@@ -992,6 +1023,7 @@ static void bpc_attrib_xattrWrite(bpc_attrib_xattr *xattr, buf_info *info)
         memcpy(info->bufP, xattr->value, xattr->valueLen);
     }
     info->bufP += xattr->valueLen;
+    info->numEntries++;
 }
 
 /*
@@ -1031,10 +1063,13 @@ uchar *bpc_attrib_file2buf(bpc_attrib_file *file, uchar *buf, uchar *bufEnd)
     }
     bufP += file->digest.len;
 
-    info.bufEnd = bufEnd;
-    info.bufP   = bufP;
+    info.bufEnd     = bufEnd;
+    info.bufP       = bufP;
+    info.numEntries = 0;
     bpc_hashtable_iterate(&file->xattrHT, (void*)bpc_attrib_xattrWrite, &info);
-
+    if ( info.numEntries != xattrEntryCnt ) {
+        bpc_logErrf("bpc_attrib_file2buf: BOTCH: wrote %u xattr entries vs %u; attrib file corrupted\n", info.numEntries, xattrEntryCnt);
+    }
     return info.bufP;
 }
 
